import * as GameTest from '@minecraft/server-gametest';
import { world, system, Vector } from '@minecraft/server';

const Core = {
    World: {
        overworld: world.getDimension('overworld'),
        print: msg => Core.World.overworld.runCommandAsync(`tellraw @a {"rawtext":[{"text":"${msg}"}]}`),
        ephemeral: async (msg,sender) => await system.run(() => sender.runCommandAsync(`tellraw @s {"rawtext":[{"text":"${msg}"}]}`))
    }
}

const { ephemeral, print } = Core.World

// Client Object
const Client = {
    
    Connect: async function(...args) {
        // Config declarations
        const DB = args[0], username = args[1], sender = args[2], event = args[3], gamemode = args[4];
        const exist = DB.findIndex(x => x.Player.name === username);
        // Fakeplayer connection check
        if (exist >= 0) return ephemeral(`§7${username} are connected`,sender)
        // Fakeplayer connection
        const FAKEPLAYER = event.spawnSimulatedPlayer(new Vector(0,4,0), username)
        FAKEPLAYER.setGameMode(gamemode == 'creative' || gamemode == 'creative' ? gamemode : 'survival');
        await FAKEPLAYER.runCommand(`tp @s "${sender.name}"`);
        FAKEPLAYER.runCommandAsync('spawnpoint @s ~~2~');
        // Pushed player to DB
        DB.push({ Player: FAKEPLAYER, Actions: [] });
    },

    Disconnect: function(...args) {
        // Config declarations
        const DB = args[0], username = args[1], sender = args[2], event = args[3];
        const exist = DB.findIndex(x => x.Player.name === username);
        // Fakeplayer disconnection check
        if (exist < 0) return ephemeral('§8You can only manipulate existing players',sender);
        // Fakeplayer disconnection
        const FAKEPLAYER = DB[exist].Player;
        FAKEPLAYER.kill();
        event.removeSimulatedPlayer(FAKEPLAYER)
        // Removed player from DB
        DB.splice(exist, 1);
    },

    Actions: function(...args) {
        // Config declarations
        const DB = args[0], username = args[1], sender = args[2], action = args[3], event = args[4];
        const exist = DB.findIndex(x => x.Player.name === username);
        // Fakeplayer disconnection check
        if (exist < 0) return ephemeral('§8You can only manipulate existing players',sender);
        // Fakeplayer action
        const FAKEPLAYER = DB[exist].Player;
        //const { x, y, z } = FAKEPLAYER.getBlockFromViewDirection()
        if (action == 'jump') FAKEPLAYER.jump();
        else if (action == 'attack') FAKEPLAYER.attack();
        else if (action == 'tp') FAKEPLAYER.runCommand(`tp @s "${sender.name}"`);
        else if (action == 'shift') FAKEPLAYER.isSneaking ? FAKEPLAYER.isSneaking = false : FAKEPLAYER.isSneaking = true;
        else if (action == 'respawn') FAKEPLAYER.respawn();
        else if (action == 'use') FAKEPLAYER.useItemInSlot(args[4] >= 0 || args[4] <= 9 ? parseInt(args[4]) : FAKEPLAYER.selectedSlot);
        else if (action == 'interact') FAKEPLAYER.interact();
        else if (action == 'rotate') FAKEPLAYER.rotateBody(parseInt(args[4]) || 20);
        // Repeat action
        else if (action == 'repeat') {
            const ACTIONS = DB[exist].Actions;
            if (['jump','attack','use','interact'].includes(args[4])) ACTIONS.push(args[4]);
            else ephemeral('§7Invalid action to repeat',sender);
        }
        // Stop action
        else if (action == 'stop') {
            DB[exist].Actions = [];
        }
    }
    
}; export { Client }